unit ActivityLog;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, ComCtrls, Dialogs, TRpcB, MFunStr, Menus;

type
  TdlgLog = class(TForm)
    btnCancel: TButton;
    pnlMain: TPanel;
    grpDates: TGroupBox;
    lblStartDate: TLabel;
    lblEndDate: TLabel;
    dtpStartDate: TDateTimePicker;
    dtpEndDate: TDateTimePicker;
    cbStartDate: TCheckBox;
    cbEndDate: TCheckBox;
    btnOK: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    procedure dtpEndDateChange(Sender: TObject);
    procedure btnShowClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbStartDateClick(Sender: TObject);
    procedure cbEndDateClick(Sender: TObject);
    procedure dtpStartDateChange(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
  private
    { Private declarations }
    function getStartDate:String;
    procedure setStartDate(StartDate:String);
    function getEndDate:String;
    procedure setEndDate(EndDate:String);
    procedure displayReport;
    function CallRPCActivityLog(StartDate,EndDate:String):TStrings;
  public
    { Public declarations }
  end;

var
  dlgLog: TdlgLog;

implementation

{$R *.DFM}

uses HepCLibrary, DisplayLog, HepC;

function TdlgLog.getStartDate:String;
begin
  if cbStartDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpStartDate.Date);
end;

procedure TdlgLog.setStartDate(StartDate:String);
begin
  if StartDate<>'' then dtpStartDate.Date:=FMDateToWindowsDate(StartDate) else if LastExtract<>'' then dtpStartDate.Date:=StrToDate(LastExtract);
end;

function TdlgLog.getEndDate:String;
begin
  if cbEndDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpEndDate.Date);
end;

procedure TdlgLog.setEndDate(EndDate:String);
begin
  if EndDate<>'' then dtpEndDate.Date:=FMDateToWindowsDate(EndDate) else dtpEndDate.Date:=Date;
end;

procedure TdlgLog.displayReport;
var
  StartDate,EndDate:String;
  Index:Integer;
  Activity,Results:TStrings;
  dlgDisplayLog:TdlgDisplayLog;
begin
  Results:=TStringList.Create;
  Results.Clear;
  dlgDisplayLog:=TdlgDisplayLog.Create(self);
  Activity:=TStringList.Create;
  HepCActivityLog.Clear;
  StartDate:=getStartDate;
  EndDate:=getEndDate;
  Results:=CallRPCActivityLog(StartDate,EndDate);
  //
  dlgDisplayLog.TreeView1.Items.Clear;
  if Results.Count>0 then
  begin
    Activity.Clear;
    for Index:=0 to Results.Count-1 do
    begin
      if Results[Index]='$$ENDOFACTIVITY' then
      begin
        dlgDisplayLog.addActivityItem(Activity);
        Activity.Clear;
      end
      else
      begin
        HepCActivityLog.Add(Results[Index]);
        Activity.Add(Results[Index]);
      end;
    end;
  end;
  //
  Activity.Free;
  Results.Free;
  dlgDisplayLog.ShowModal;
  dlgDisplayLog.Destroy;
end;

function TdlgLog.CallRPCActivityLog(StartDate,EndDate:String):TStrings;
begin
  Result:=TStringList.Create;
  Result.Clear;
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR ACTIVITY LOG';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    Param[1].PType:=Literal;
    Param[1].Value:=StartDate;
    Param[2].PType:=Literal;
    Param[2].Value:=EndDate;
    try Call;
      if Results.Count>0 then
      begin
        Result.AddStrings(Results);
      end
      else displayError('No activity to report for specified date range.',mtInformation);
    except on EBrokerError do displayError('Broker Error: Failed to retrieve activity log from VistA',mtError);
    end;
  end;
end;

procedure TdlgLog.dtpStartDateChange(Sender: TObject);
begin
  if (not cbStartDate.Checked) and (getEndDate<getStartDate) then
  begin
    displayError('End date cannot be before start date',mtError);
    setStartDate(getEndDate);
  end;
end;

procedure TdlgLog.dtpEndDateChange(Sender: TObject);
begin
  if (not cbStartDate.Checked) and (getEndDate<getStartDate) then
  begin
    displayError('End date cannot be before start date',mtError);
    setEndDate(getStartDate);
  end;
end;

procedure TdlgLog.btnShowClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TdlgLog.btnPrintClick(Sender: TObject);
begin
  ModalResult:=mrOK;
end;

procedure TdlgLog.FormCreate(Sender: TObject);
begin
  setEndDate('');
  setStartDate('');
end;

procedure TdlgLog.cbStartDateClick(Sender: TObject);
begin
  dtpStartDate.Enabled:=(not cbStartDate.Checked);
end;

procedure TdlgLog.cbEndDateClick(Sender: TObject);
begin
  dtpEndDate.Enabled:=(not cbEndDate.Checked);
end;


procedure TdlgLog.btnOKClick(Sender: TObject);
begin
  displayReport;
end;

procedure TdlgLog.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgLog.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

end.
