unit DisplayLog;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, TRpcB,
  Buttons, ExtCtrls, ComCtrls, MFunStr, Dialogs, ImgList,
  HepCLibrary, Menus;

type
  TdlgDisplayLog = class(TForm)
    btnOK: TButton;
    StatusBar1: TStatusBar;
    ImageList1: TImageList;
    Bevel1: TBevel;
    pnlPanels: TPanel;
    Splitter1: TSplitter;
    pnlMessage: TPanel;
    lblHeading: TLabel;
    pnlAddInfo: TPanel;
    lblAddInfo: TLabel;
    edtAddInfo: TMemo;
    pnlMessInfo: TPanel;
    lblType: TLabel;
    lblDateTime: TLabel;
    lblPatient: TLabel;
    edtMessage: TEdit;
    edtDateTime: TEdit;
    edtPatient: TEdit;
    pnlActivity: TPanel;
    lblHeader: TLabel;
    pnlPatientRate: TPanel;
    lblComment: TLabel;
    edtPatients: TEdit;
    edtRate: TEdit;
    edtErrors: TEdit;
    pnlMessages: TPanel;
    lblMessages: TLabel;
    edtMessages: TMemo;
    pnlRegistries: TPanel;
    lblRegistries: TLabel;
    edtRegistries: TMemo;
    pnlInfo: TPanel;
    lblActivity: TLabel;
    lblJob: TLabel;
    lblStartDate: TLabel;
    lblEndDate: TLabel;
    edtActivity: TEdit;
    edtJob: TEdit;
    edtStartDate: TEdit;
    edtEndDate: TEdit;
    TreeView1: TTreeView;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    procedure TreeView1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure TreeView1Expanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeView1Collapsing(Sender: TObject; Node: TTreeNode;
      var AllowCollapse: Boolean);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure clearFields;
    procedure addActivityItem(Activity:TStrings);
    procedure displayActivity(IEN:String);
    procedure displayMessage(IEN:String);
  end;

var
  dlgDisplayLog: TdlgDisplayLog;

implementation

uses HepC;

{$R *.DFM}

procedure TdlgDisplayLog.clearFields;
begin
  edtRegistries.Clear;
  edtMessages.Clear;
  edtAddInfo.Clear;
  pnlAddInfo.Visible:=False;
  pnlPatientRate.Visible:=False;
  pnlActivity.Visible:=False;
  pnlMessage.Visible:=False;
end;

procedure TdlgDisplayLog.displayMessage(IEN:String);
var
  Index:Integer;
begin
  clearFields;
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR LOG MESSAGE DETAILS';
    Param[0].PType:=Literal;
    Param[0].Value:=IEN;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='1' then
        begin
          pnlMessage.Visible:=True;
          lblType.Caption:=' '+Results[1]+': ';
          edtMessage.Text:=Results[2];
          edtDateTime.Text:=Results[3];
          edtPatient.Text:=Results[4];
          edtPatient.Visible:=(Results[4]<>'');
          lblPatient.Visible:=(Results[4]<>'');
          Index:=5;
          while (Index<Results.Count) do
          begin
            edtAddInfo.Lines.Add(Results[Index]);
            Index:=Index+1;
          end;
          pnlAddInfo.Visible:=(edtAddInfo.Lines.Count>0);
        end
        else displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: Failed to retrieve message details from VistA',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to retrieve message details from VistA',mtError);
    end;
  end;
end;

procedure TdlgDisplayLog.displayActivity(IEN:String);
var
  Index:Integer;
begin
  clearFields;
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR ACTIVITY DETAILS';
    Param[0].PType:=Literal;
    Param[0].Value:=IEN;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='1' then
        begin
          pnlActivity.Visible:=True;
          edtActivity.Text:=Results[1];
          edtJob.Text:=Results[2];
          edtStartDate.Text:=Results[3];
          edtEndDate.Text:=Results[4];
          Index:=5;
          if Index<Results.Count then
            if Results[Index]='$$STARTREGISTRYLIST' then
            begin
              Index:=Index+1;
              while Results[Index]<>'$$ENDREGISTRYLIST' do
              begin
                edtRegistries.Lines.Add(Results[Index]);
                Index:=Index+1;
              end;
            end;
          Index:=Index+1;
          if Index<Results.Count then
            if Results[Index]='$$STARTMESSAGELIST' then
            begin
              Index:=Index+1;
              while Results[Index]<>'$$ENDMESSAGELIST' do
              begin
                edtMessages.Lines.Add(Results[Index]);
                Index:=Index+1;
              end;
            end;
          Index:=Index+1;
          if Index<Results.Count then
            if Results[Index]='$$STARTADDINFO' then
            begin
              pnlPatientRate.Visible:=True;
              edtPatients.Text:=Results[Index+1]+' patients were processed';
              edtRate.Text:='at a rate of '+Results[Index+3]+' per second.';
              edtErrors.Text:='There were errors with '+Results[Index+2]+' patients.';
            end;
        end
        else displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: Failed to retrieve activity details from VistA',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to retrieve activity details from VistA',mtError);
    end;
  end;
end;

procedure TdlgDisplayLog.addActivityItem(Activity:TStrings);
var
  Index:Integer;
  MyTreeNode,MyChildNode:TTreeNode;
  TreeCap:String;
begin
  TreeCap:=Piece(Activity[0],'^',3)+' ('+Piece(Activity[0],'^',2)+')';
  with TreeView1.Items do
  begin
    MyTreeNode:=Add(nil,TreeCap);
    MyTreeNode.ImageIndex:=0;
    MyTreeNode.SelectedIndex:=0;
    for Index:=1 to Activity.Count-1 do
    begin
      MyChildNode:=AddChild(MyTreeNode,Piece(Activity[Index],'^',3));
      case StrToInt(Piece(Activity[Index],'^',2)) of
        1: MyChildNode.ImageIndex:=5;
        2: MyChildNode.ImageIndex:=6;
        3: MyChildNode.ImageIndex:=8;
        4: MyChildNode.ImageIndex:=5;
        5: MyChildNode.ImageIndex:=7;
        6: MyChildNode.ImageIndex:=4;
      end;
      MyChildNode.SelectedIndex:=MyChildNode.ImageIndex;
    end;
  end;
end;

procedure TdlgDisplayLog.TreeView1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  TreeNode:TTreeNode;
  HT:THitTests;
begin
  if (Sender is TTreeView) then
  begin
    with Sender as TTreeView do
    begin
      HT:=GetHitTestInfoAt(X,Y);
      if (htOnItem in HT) then
      begin
        TreeNode:=GetNodeAt(X,Y);
        if TreeNode.Level=0 then displayActivity(Piece(HepCActivityLog[TreeNode.AbsoluteIndex],'^',4));
        if TreeNode.Level=1 then displayMessage(Piece(HepCActivityLog[TreeNode.AbsoluteIndex],'^',4));
      end;
    end;
  end;
end;

procedure TdlgDisplayLog.FormShow(Sender: TObject);
begin
  clearFields;
end;

procedure TdlgDisplayLog.TreeView1Expanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
  Node.ImageIndex:=1;
  Node.SelectedIndex:=1;
end;

procedure TdlgDisplayLog.TreeView1Collapsing(Sender: TObject;
  Node: TTreeNode; var AllowCollapse: Boolean);
begin
  Node.ImageIndex:=0;
  Node.SelectedIndex:=0;
end;

procedure TdlgDisplayLog.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgDisplayLog.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

end.
