unit DrugActivityDates;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, HepCLibrary, Dialogs, Printers, TRpcB,
  FileCtrl, MFunStr, IniFiles, HepCCommon, Menus;

type
  TdlgDrugDates = class(TForm)
    PrintDialog1: TPrintDialog;
    SaveDialog1: TSaveDialog;
    PageControl1: TPageControl;
    tabCreate: TTabSheet;
    tabStatus: TTabSheet;
    pnlMain: TPanel;
    pnlDates: TPanel;
    grpDates: TGroupBox;
    lblStartDate: TLabel;
    lblEndDate: TLabel;
    dtpStartDate: TDateTimePicker;
    dtpEndDate: TDateTimePicker;
    cbStartDate: TCheckBox;
    cbEndDate: TCheckBox;
    pnlTimePeriod: TPanel;
    grpTimePeriod: TRadioGroup;
    cmbQuarterly: TComboBox;
    pnlYear: TPanel;
    LeftLine: TBevel;
    lblYear: TLabel;
    cmbYear: TComboBox;
    pnlDateRange: TPanel;
    grpDateRange: TRadioGroup;
    btnOK: TButton;
    tabPrint: TTabSheet;
    pnlOptions: TPanel;
    pnlButtons: TPanel;
    pnlStatus: TPanel;
    btnCancel: TButton;
    grpStatus: TGroupBox;
    lblTask: TLabel;
    lblStatus: TLabel;
    Timer1: TTimer;
    lblRefresh: TLabel;
    btnExport: TBitBtn;
    btnShow: TBitBtn;
    btnPrint: TBitBtn;
    edtTask: TEdit;
    edtStatus: TEdit;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    SaveReport1: TMenuItem;
    DisplayReport1: TMenuItem;
    PrintReport1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    btnAbort: TButton;
    Animate1: TAnimate;
    procedure grpDateRangeClick(Sender: TObject);
    procedure cbStartDateClick(Sender: TObject);
    procedure cbEndDateClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnShowClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure PageControl1Change(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure grpTimePeriodClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
    procedure btnAbortClick(Sender: TObject);
  private
    { Private declarations }
    function getStartDate:String;
    procedure setStartDate(StartDate:String);
    function getEndDate:String;
    procedure setEndDate(EndDate:String);
    function getDateRange:Integer;
    procedure setDateRange(DateRange:Integer);
    function CallRPCReportText:TStrings;
    procedure CallRPCGetHandle(var Hndl:TRPCHandle);
    procedure CallRPCTaskReport(StartDate,EndDate:String);
    //procedure CallRPCStoreHandle;
    procedure ExportReport(FileName:String);
    procedure DisplayReport;
    procedure UnloadIniFile;
    procedure UpdateIniFile;
  public
    { Public declarations }
    procedure printHeader(var Line:Integer);
    procedure ReportLineToCanvas(var Line:Integer;DataString:String;Bold:Boolean);
    procedure getStartEndDates(var StartDate:String;var EndDate:String);
  end;

var
  dlgDrugDates: TdlgDrugDates;
  RPCHandle:TRPCHandle;

implementation

uses HepC, DrugReport;

{$R *.DFM}

procedure TdlgDrugDates.UnloadIniFile;
var
  S:String;
begin
  S:=GetGUISettings(frmHepC.RPC,'dlgDrugDates');
  if Piece(S,';',1)<>'' then setDateRange(StrToInt(Piece(S,';',1))) else setDateRange(0);
  setEndDate(Piece(S,';',2));
  setStartDate(Piece(S,';',3));
end;

procedure TdlgDrugDates.UpdateIniFile;
var
  S:String;
begin
  S:=IntToStr(getDateRange)+';'+getEndDate+';'+getStartDate;
  SetGUISettings(frmHepC.RPC,'dlgDrugDates',S);
end;

function TdlgDrugDates.getStartDate:String;
begin
  if cbStartDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpStartDate.Date);
end;

procedure TdlgDrugDates.setStartDate(StartDate:String);
begin
  if StartDate<>'' then dtpStartDate.Date:=FMDateToWindowsDate(StartDate) else
  begin
    try dtpStartDate.Date:=StrToDate(LastExtract);
    except on EConvertError do;
    end;
    cbStartDate.Checked:=True;
  end;
end;

function TdlgDrugDates.getEndDate:String;
begin
  if cbEndDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpEndDate.Date);
end;

procedure TdlgDrugDates.setEndDate(EndDate:String);
begin
  if EndDate<>'' then dtpEndDate.Date:=FMDateToWindowsDate(EndDate) else
  begin
    dtpEndDate.Date:=Date;
    cbEndDate.Checked:=True;
  end;
end;

procedure TdlgDrugDates.getStartEndDates(var StartDate:String;var EndDate:String);
var
  Year:Integer;
begin
  if grpDateRange.ItemIndex=0 then
  begin
    StartDate:=GetStartDate;
    EndDate:=GetEndDate;
  end
  else
  begin
    Year:=StrToInt(cmbYear.Items[cmbYear.ItemIndex]);
    case grpTimePeriod.ItemIndex of
    0:
      begin
        StartDate:=ConstructDate(Year,1,1);
        EndDate:=ConstructDate(Year,12,31);
      end;
    1:
      begin
        StartDate:=ConstructDate(Year-1,10,1);
        EndDate:=ConstructDate(Year,9,30);
      end;
    2:
      begin
        case cmbQuarterly.ItemIndex of
        0:
          begin
            StartDate:=ConstructDate(Year,1,1);
            EndDate:=ConstructDate(Year,3,31);
          end;
        1:
          begin
            StartDate:=ConstructDate(Year,4,1);
            EndDate:=ConstructDate(Year,6,30);
          end;
        2:
          begin
            StartDate:=ConstructDate(Year,7,1);
            EndDate:=ConstructDate(Year,9,30);
          end;
        3:
          begin
            StartDate:=ConstructDate(Year,10,1);
            EndDate:=ConstructDate(Year,12,31);
          end;
        end;
      end;
    end;
  end;
end;

function TdlgDrugDates.getDateRange:Integer;
begin
  Result:=grpDateRange.ItemIndex;
end;

procedure TdlgDrugDates.setDateRange(DateRange:Integer);
begin
  grpDateRange.ItemIndex:=DateRange;
end;

procedure TdlgDrugDates.grpDateRangeClick(Sender: TObject);
begin
  if grpDateRange.ItemIndex=0 then
  begin
    pnlTimePeriod.Visible:=False;
    pnlDates.Top:=37;
    pnlDates.Visible:=True;
  end
  else
  begin
    pnlDates.Visible:=False;
    pnlDates.Top:=37;
    pnlTimePeriod.Visible:=True;
  end;
end;

procedure TdlgDrugDates.cbStartDateClick(Sender: TObject);
begin
  dtpStartDate.Enabled:=(not cbStartDate.Checked);
end;

procedure TdlgDrugDates.cbEndDateClick(Sender: TObject);
begin
  dtpEndDate.Enabled:=(not cbEndDate.Checked);
end;

procedure TdlgDrugDates.ReportLineToCanvas(var Line:Integer;DataString:String;Bold:Boolean);
begin
  //put blank line in before each patient
  if Piece(DataString,'^',1)<>'' then
  begin
    Line:=Line+1;
    if ((80*Line)>Printer.PageHeight) then
    begin
      Printer.NewPage;
      printHeader(Line);
    end;
  end;
  if Bold then Printer.Canvas.Font.Style:=[fsBold];
  Printer.Canvas.TextOut(0,80*(Line-1),Piece(DataString,'^',1));
  Printer.Canvas.TextOut(130,80*(Line-1),Copy(Piece(DataString,'^',2),1,35));
  Printer.Canvas.TextOut(1360,80*(Line-1),Piece(DataString,'^',3));
  Printer.Canvas.TextOut(1680,80*(Line-1),Copy(Piece(DataString,'^',4),1,35));
  Printer.Canvas.TextOut(3240,80*(Line-1),Piece(DataString,'^',5));
  Printer.Canvas.TextOut(3760,80*(Line-1),Piece(DataString,'^',6));
  if Bold then Printer.Canvas.Font.Style:=[];
  Line:=Line+1;
  if ((80*Line)>Printer.PageHeight) then
  begin
    Printer.NewPage;
    printHeader(Line);
  end;
end;

procedure TdlgDrugDates.printHeader(var Line:Integer);
var
  StartDate,EndDate:String;
  str:String;
  chrw,chrh:Integer;
begin
  //GetStartEndDates(StartDate,EndDate);
  StartDate:=Piece(DrugReportData[0],'^',1);
  EndDate:=Piece(DrugReportData[0],'^',2);
  //
  str:='Hepatitis C - Rx Report';
  chrw:=Printer.Canvas.TextExtent(str).cx;
  chrh:=Printer.Canvas.TextExtent(str).cy;
  Printer.Canvas.Font.Style:=[fsBold,fsUnderline];
  Printer.Canvas.TextOut((Printer.PageWidth-chrw) div 2,0,str);
  Printer.Canvas.Font.Style:=[];
  str:='Page: '+IntToStr(Printer.PageNumber);
  chrw:=Printer.Canvas.TextExtent(str).cx;
  Printer.Canvas.TextOut(Printer.PageWidth-chrw,0,str);
  //
  if Printer.PageNumber=1 then
  begin
    str:='Start Date: ';
    if StartDate<>'' then str:=str+DateToStr(FMDateToWindowsDate(StartDate)) else str:=str+' None';
    Printer.Canvas.TextOut(0,chrh*2,str);
    str:='Report Created: '+DateToStr(Date);
    chrw:=Printer.Canvas.TextExtent(str).cx;
    Printer.Canvas.TextOut(Printer.PageWidth-chrw,chrh*2,str);
    //
    str:='End Date: ';
    if EndDate<>'' then str:=str+DateToStr(FMDateToWindowsDate(EndDate)) else str:=str+' None';
    Printer.Canvas.TextOut(0,chrh*3,str);
    //
    Line:=5;
  end
  else Line:=2;
  ReportLineToCanvas(Line,'^NAME^LAST^DRUG^NDC^FILL',True);
  ReportLineToCanvas(Line,'^^FOUR^NAME^^DATE',True);
end;

procedure TdlgDrugDates.CallRPCTaskReport(StartDate,EndDate:String);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='XWB DEFERRED RPC';
    Param[0].PType:=Literal;
    Param[0].Value:='ROR DRUG REPORT';
    Param[1].PType:=Literal;
    Param[1].Value:=RegistryName;
    Param[2].PType:=Literal;
    Param[2].Value:=StartDate;
    Param[3].PType:=Literal;
    Param[3].Value:=EndDate;
    try Call;
    if Results.Count>0 then
    begin
      if Piece(Results[0],'^',1)<>'-1' then
      begin
        RPCHandle.ID:=Results[0];
        RPCHandle.TaskStatus:='Placed on TaskMan queue';
      end
      else displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
    end
    else displayError('Broker Error: Failed to schedule report',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to schedule report',mtError);
    end;
    if RPCHandle.ID<>'' then RPCHandle.CallRPCStoreHandle('16');
  end;
end;

procedure TdlgDrugDates.CallRPCGetHandle(var Hndl:TRPCHandle);
begin
  Hndl.CallRPCGetHandle('16');
  edtStatus.Text:=Hndl.TaskStatus;
  edtTask.Text:=Hndl.TaskID;
  btnExport.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  SaveReport1.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  btnShow.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  DisplayReport1.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  btnPrint.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  PrintReport1.Enabled:=(Hndl.TaskStatus='Completed Successfully');
  btnOK.Enabled:=((Hndl.TaskStatus<>'Placed on TaskMan queue') and (Hndl.TaskStatus<>'Running'));
  lblRefresh.Visible:=((Hndl.TaskStatus='Placed on TaskMan queue') or (Hndl.TaskStatus='Running'));
  btnAbort.Enabled:=((Hndl.TaskStatus='Placed on TaskMan queue') or (Hndl.TaskStatus='Running'));
  Animate1.Active:=(Hndl.TaskStatus='Running');
end;

function TdlgDrugDates.CallRPCReportText:TStrings;
var
  cnt,i:Integer;
begin
  cnt:=0;
  Result:=TStringList.Create;
  Result.Clear;
  CallRPCGetHandle(RPCHandle);
  if ((RPCHandle.ID<>'') and (RPCHandle.TaskStatus='Completed Successfully')) then
  begin
    with frmHepC.RPC do
    begin
      RemoteProcedure:='XWB DEFERRED GETDATA';
      Param[0].PType:=Literal;
      Param[0].Value:=RPCHandle.ID;
      try Call;
        Result.Add(Results[1]);
        if Results.Count>2 then
        begin
          for i:=2 to Results.Count-1 do
          begin
            if Piece(Results[i],'^',1)<>'' then
            begin
              cnt:=cnt+1;
              Result.Add(IntToStr(cnt)+'^'+Results[i]);
            end
            else Result.Add('^'+Results[i]);
          end;
        end
        else Result.Add('^Nothing to report.');
      except on EBrokerError do displayError('Broker Error: Failed to retrieve report info',mtError);
      end;
    end;
  end
  else if ((RPCHandle.ID<>'') and (RPCHandle.TaskStatus='Running')) then displayError('Previously scheduled report still running',mtError)
  else displayError('Cannot find previous report',mtError);
end;

procedure TdlgDrugDates.ExportReport(FileName:String);
var
  SaveText:TextFile;
  str:String;
  Index,i:Integer;
  ReportText:TStrings;
begin
  ReportText:=TStringList.Create;
  ReportText.Clear;
  DrugReportData:=CallRPCReportText;
  //
  ReportText.Add(#9+'NAME'+#9+'LAST FOUR'+#9+'DRUG NAME'+#9+'NDC'+#9+'FILL DATE');
  for Index:=0 to DrugReportData.Count-1 do
  begin
    str:='';
    for i:=1 to 5 do str:=str+Piece(DrugReportData[Index],'^',i)+#9;
    str:=str+Piece(DrugReportData[Index],'^',6);
    ReportText.Add(str);
  end;
  AssignFile(SaveText,FileName);
  Rewrite(SaveText);
  Write(SaveText,ReportText.Text);
  CloseFile(SaveText);
  //
  ReportText.Destroy;
end;

procedure TdlgDrugDates.DisplayReport;
var
  Index:Integer;
  SDate,EDate:String;
  dlgDrugReport:TdlgDrugReport;
begin
  //getStartEndDates(SDate,EDate);
  DrugReportData:=CallRPCReportText;
  dlgDrugReport:=TdlgDrugReport.Create(self);
  SDate:=Piece(DrugReportData[0],'^',1);
  dlgDrugReport.setStartDate(SDate);
  EDate:=Piece(DrugReportData[0],'^',2);
  dlgDrugReport.setEndDate(EDate);
  //
  dlgDrugReport.listReport.Items.Clear;
  for Index:=1 to DrugReportData.Count-1 do
  begin
    dlgDrugReport.setReportItem(DrugReportData[Index]);
  end;
  //
  dlgDrugReport.ShowModal;
  dlgDrugReport.Destroy;
end;

procedure TdlgDrugDates.btnExportClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    displayError('End date cannot be before start date',mtError);
    dtpStartDate.SetFocus;
    Exit;
  end;
  if SaveDialog1.Execute then
  begin
    if not DirectoryExists(ExtractFilePath(SaveDialog1.FileName)) then ForceDirectories(ExtractFilePath(SaveDialog1.FileName));
    ExportReport(SaveDialog1.FileName);
  end;
end;

procedure TdlgDrugDates.btnShowClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    displayError('End date cannot be before start date',mtError);
    dtpStartDate.SetFocus;
    Exit;
  end;
  DisplayReport;
end;

procedure TdlgDrugDates.btnPrintClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    displayError('End date cannot be before start date',mtError);
    dtpStartDate.SetFocus;
    Exit;
  end;
  if PrintDialog1.Execute then
  begin
    DrugReportData:=CallRPCReportText;
    PrintDrugReport;
  end;
end;

procedure TdlgDrugDates.FormShow(Sender: TObject);
var
  Yr,Mn,Dy:Word;
  i:Integer;
begin
  setEndDate(WindowsDateToFMDate(Date));
  //Default Date & Time Pickers
  dtpStartDate.MaxDate:=Date;
  //Populate Year ComboBox
  DecodeDate(Date,Yr,Mn,Dy);
  for i:=1990 to Yr+100 do cmbYear.Items.Add(IntToStr(i));
  cmbYear.ItemIndex:=cmbYear.Items.IndexOf(IntToStr(Yr));
  CallRPCGetHandle(RPCHandle);
end;

procedure TdlgDrugDates.btnOKClick(Sender: TObject);
var
  StartDate,EndDate:String;
begin
  GetStartEndDates(StartDate,EndDate);
  CallRPCTaskReport(StartDate,EndDate);
  btnOK.Enabled:=False;
end;

procedure TdlgDrugDates.FormCreate(Sender: TObject);
begin
  PageControl1.ActivePageIndex:=0;
  RPCHandle:=TRPCHandle.Create;
  self.UnloadIniFile;
end;

procedure TdlgDrugDates.FormDestroy(Sender: TObject);
begin
  RPCHandle.Destroy;
end;

procedure TdlgDrugDates.PageControl1Change(Sender: TObject);
begin
  if PageControl1.ActivePageIndex>0 then CallRPCGetHandle(RPCHandle);
  Timer1.Enabled:=((PageControl1.ActivePageIndex=1) and (RPCHandle.TaskStatus<>'Done'));
  btnCancel.SetFocus;
end;

procedure TdlgDrugDates.Timer1Timer(Sender: TObject);
begin
  CallRPCGetHandle(RPCHandle);
end;

procedure TdlgDrugDates.grpTimePeriodClick(Sender: TObject);
var
  Dy,Mn,Yr:Word;
begin
  if grpTimePeriod.ItemIndex=2 then
  begin
    DecodeDate(Date,Yr,Mn,Dy);
    case Mn of
      1..3: cmbQuarterly.ItemIndex:=0;
      4..6: cmbQuarterly.ItemIndex:=1;
      7..9: cmbQuarterly.ItemIndex:=2;
      10..12: cmbQuarterly.ItemIndex:=3;
    end;
    cmbQuarterly.Enabled:=True;
    cmbQuarterly.SetFocus;
  end
  else cmbQuarterly.Enabled:=False;
end;

procedure TdlgDrugDates.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  self.UpdateIniFile;
end;

procedure TdlgDrugDates.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgDrugDates.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

procedure TdlgDrugDates.btnAbortClick(Sender: TObject);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR CLEAR RPC HANDLE';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    Param[1].PType:=Literal;
    Param[1].Value:='16';
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='-1' then displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: Failed to abort report process',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to abort report process',mtError);
    end;
  end;
  CallRPCGetHandle(RPCHandle);
end;

end.
