unit HepCCommon;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Dialogs,
     Buttons, ExtCtrls, ComCtrls, PatientSearch, MFunStr, HepCLibrary, Printers,
     TrpcB;

type
  TRPCHandle = class(TObject)
  private
    FID:String;
    FTaskStatus:String;
    FTaskID:String;
  public
    constructor Create;
    destructor Destroy;override;
    procedure CallRPCGetHandle(RORFLD:String);
    procedure CallRPCStoreHandle(RORFLD:String);
  published
    property ID:String
      read FID write FID;
    property TaskStatus:String
      read FTaskStatus write FTaskStatus;
    property TaskID:String
      read FTaskID write FTaskID;
  end;

function getPatientIEN:String;
function getPatientName:String;
function getPatientSSN:String;
function getPatientLastFour:String;
function getPatientDOB:String;
function getSelectionRules(listInfo:TListView):TStrings;
procedure setPatientName(Edit:TEdit);
procedure setPatientSSN(Edit:TEdit);
procedure setPatientDOB(Edit:TEdit);
procedure setSelectionRule(listInfo:TListView;Rule:String);
procedure PrintReport;
procedure PrintDrugReport;

implementation

uses HepC;

constructor TRPCHandle.Create;
begin
  inherited Create;
  FID:='';
  FTaskStatus:='BadHandle';
  FTaskID:='';
end;

destructor TRPCHandle.Destroy;
begin
  inherited Destroy;
end;

procedure TRPCHandle.CallRPCGetHandle(RORFLD:String);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR GET RPC HANDLE';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    Param[1].PType:=Literal;
    Param[1].Value:=RORFLD;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)<>'-1' then
        begin
          self.ID:=Results[0];
          if Results[1]='Done' then self.TaskStatus:='Completed Successfully'
          else if Results[1]='New' then self.TaskStatus:='Placed on TaskMan queue'
          else self.TaskStatus:=Results[1];
          self.TaskID:=Results[2];
        end
        else displayError('Broker Error:'+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: Failed to retrieve Handle',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to retrieve Handle',mtError);
    end;
  end;
end;

function getPatientIEN:String;
begin
  Result:=dlgPatientSearch.SelectedIEN;
end;

function getPatientName:String;
begin
  Result:=dlgPatientSearch.SelectedName;
end;

function getPatientSSN:String;
begin
  Result:=dlgPatientSearch.SelectedSSN;
end;

function getPatientLastFour:String;
begin
  Result:=Copy(getPatientSSN,6,5);
end;

function getPatientDOB:String;
begin
  Result:=dlgPatientSearch.SelectedDOB;
end;

function getSelectionRules(listInfo:TListView):TStrings;
var
  i:Integer;
begin
  Result:=TStringList.Create;
  for i:=0 to listInfo.Items.Count-1 do
  begin
    Result.Add(listInfo.Items[i].SubItems[1]+'^'+WindowsDateToFMDate(StrToDate(listInfo.Items[i].SubItems[0])));
  end;
end;

procedure setPatientName(Edit:TEdit);
begin
  Edit.Text:=getPatientName;
end;

procedure setPatientSSN(Edit:TEdit);
var
  SSN:String;
begin
  Edit.Text:='';
  SSN:=getPatientSSN;
  if SSN<>'' then Edit.Text:=Copy(SSN,1,3)+'-'+Copy(SSN,4,2)+'-'+Copy(SSN,6,Length(SSN)-5);
end;

procedure setPatientDOB(Edit:TEdit);
begin
  Edit.Text:=getPatientDOB;
end;

procedure setSelectionRule(listInfo:TListView;Rule:String);
var
  listitem:TListItem;
begin
  listitem:=listInfo.Items.Add;
  listitem.Caption:=Piece(Rule,'^',1);
  listitem.SubItems.Add(DateToStr(FMDateToWindowsDate(Piece(Rule,'^',2))));
  listitem.SubItems.Add(Piece(Rule,'^',3));
end;

procedure PrintReport;
var
  Index,Line,TotalCols:Integer;
  AddFlds:String;
begin
  TotalCols:=0;
  AddFlds:=dlgLocRegPatRep.GetAdditionalFields;
  for Index:=1 to 8 do
  begin
    if Piece(AddFlds,'^',Index)='1' then TotalCols:=TotalCols+1;
    if ((Index=6) and (Piece(AddFlds,'^',Index)='1')) then TotalCols:=TotalCols+2;
  end;
  if Piece(AddFlds,'^',9)='1' then TotalCols:=TotalCols+2;
  if Printer.Orientation=poPortrait then
    if ((1680+(520*TotalCols))>Printer.PageWidth) then
    begin
      if MessageDlg('Report may exceed printer margins. Print landscape instead?',mtConfirmation,[mbYes,mbNo],0)=mrYes then
        Printer.Orientation:=poLandScape;
    end;
  //
  Printer.Title:='Local Registry';
  Printer.Canvas.Font.Size:=8;
  Printer.BeginDoc;
  dlgLocRegPatRep.printHeader(Line);
  for Index:=0 to ReportData.Count-1 do
  begin
    dlgLocRegPatRep.ReportLineToCanvas(Line,ReportData[Index],AddFlds,False);
  end;
  Printer.EndDoc;
  displayError(Printer.Title+' sent to printer.',mtInformation);
end;

procedure PrintDrugReport;
var
  Index,Line:Integer;
begin
  Printer.Title:='Hepatitis C Drug Therapy List';
  Printer.Canvas.Font.Size:=8;
  Printer.BeginDoc;
  dlgDrugDates.printHeader(Line);
  for Index:=1 to DrugReportData.Count-1 do
  begin
    dlgDrugDates.ReportLineToCanvas(Line,DrugReportData[Index],False);
  end;
  Printer.EndDoc;
  displayError(Printer.Title+' sent to printer.',mtInformation);
end;

procedure TRPCHandle.CallRPCStoreHandle(RORFLD:String);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR STORE RPC HANDLE';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    Param[1].PType:=Literal;
    Param[1].Value:=self.ID;
    Param[2].PType:=Literal;
    Param[2].Value:=RORFLD;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='-1' then displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: Failed to store Handle',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to store Handle',mtError);
    end;
  end;
end;

end.
