unit ManIDrug;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, HepCLibrary, MFunStr, TRpcB,
  Dialogs, HepCCommon, Menus;

type
  TdlgManIDrug = class(TForm)
    sbMain: TStatusBar;
    btnCancel: TButton;
    btnOK: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    pnlMain: TPanel;
    pnlPat: TPanel;
    pnlPatDetails: TPanel;
    edtSSN: TEdit;
    edtDOB: TEdit;
    edtName: TEdit;
    pnlDetails: TPanel;
    grpInfo: TGroupBox;
    edtMessage: TMemo;
    listInfo: TListView;
    grpDrugs: TGroupBox;
    cbIDrugs: TCheckBox;
    grpWarning: TGroupBox;
    cbWarning: TCheckBox;
    procedure btnOKClick(Sender: TObject);
    procedure cbIDrugsClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbWarningClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    function getIDrugs:Integer;
    procedure setIDrugs(IDrug:Boolean);
    function getWarning:Integer;
    procedure setWarning(Warning:Boolean);
    procedure setPrescriptionDrug(Drug:String);
    procedure ClearAllFields;
    procedure isModified;
  public
    { Public declarations }
    function CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
    procedure UpdateIDrugs(RPC:TRPCBroker);
  end;

var
  dlgManIDrug: TdlgManIDrug;
  Modified, OriginalIDrugs, OriginalInProfile: Boolean;

implementation

uses PatientSearch, Wait, HepC;

{$R *.DFM}

function TdlgManIDrug.getIDrugs:Integer;
begin
  if cbIDrugs.Checked then Result:=1 else Result:=0;
end;

procedure TdlgManIDrug.setIDrugs(IDrug:Boolean);
begin
  cbIDrugs.Checked:=IDrug;
  OriginalIDrugs:=cbIDrugs.Checked;
  grpWarning.Visible:=IDrug;
end;

function TdlgManIDrug.getWarning:Integer;
begin
  if cbWarning.Checked then Result:=1 else Result:=0;
end;

procedure TdlgManIDrug.setWarning(Warning:Boolean);
begin
  cbWarning.Checked:=Warning;
  OriginalInProfile:=cbWarning.Checked;
end;

procedure TdlgManIDrug.setPrescriptionDrug(Drug:String);
var
  listitem:TListItem;
begin
  listitem:=listInfo.Items.Add;
  listitem.Caption:=Piece(Drug,'^',2);
  listitem.SubItems.Add(Piece(Drug,'^',1));
end;

procedure TdlgManIDrug.ClearAllFields;
begin
  setIDrugs(False);
  setWarning(False);
  listInfo.Items.Clear;
  edtMessage.Visible:=False;
end;

procedure TdlgManIDrug.isModified;
begin
  Modified:=False;
  if OriginalIDrugs<>cbIDrugs.Checked then Modified:=True;
  if cbIDrugs.Checked then if OriginalInProfile<>cbWarning.Checked then Modified:=True;
  if Modified then sbMain.Panels[0].Text:='Modified' else sbMain.Panels[0].Text:='';
end;

function TdlgManIDrug.CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
var
  i:Integer;
begin
  Result:=True;
  dlgWait.StartStop(True);
  ClearAllFields;
  RPC.RemoteProcedure:='ROR INV DRUGS';
  RPC.Param[0].PType:=Literal;
  RPC.Param[0].Value:=RegistryName;
  RPC.Param[1].PType:=Literal;
  RPC.Param[1].Value:=Patien;
  try RPC.Call;
    if RPC.Results.Count>0 then
    begin
      if Piece(RPC.Results[0],'^',1)='1' then
      begin
        if Piece(RPC.Results[0],'^',2)='1' then
        begin
          setIDrugs(True);
          if Piece(RPC.Results[0],'^',3)='1' then setWarning(True) else setWarning(False);
        end
        else
        begin
          setIDrugs(False);
          setWarning(False);
        end;
        if RPC.Results.Count>1 then
        begin
          listInfo.Visible:=True;
          edtMessage.Visible:=False;
          for i:=1 to RPC.Results.Count-1 do
          begin
            setPrescriptionDrug(RPC.Results[i]);
          end;
        end
        else
        begin
          listInfo.Visible:=False;
          edtMessage.Visible:=True;
        end;
      end
      else
      begin
        displayError('Broker Error: '+Piece(RPC.Results[0],'^',2),mtError);
        Result:=False;
      end;
    end
    else
    begin
      displayError('Broker Error: Failed to retrieve info from VistA',mtError);
      Result:=False;
    end;
  except on EBrokerError do
  begin
    displayError('Broker Error: Failed to retrieve info from VistA',mtError);
    Result:=False;
  end;
  end;
  dlgWait.StartStop(False);
end;

procedure TdlgManIDrug.UpdateIDrugs(RPC:TRPCBroker);
begin
  with RPC do
  begin
    RemoteProcedure:='ROR IDRUGS PATIENT SET';
    Param[0].PType:=Literal;
    Param[0].Value:=getPatientIEN;
    Param[1].PType:=Literal;
    Param[1].Value:=RegistryName;
    Param[2].PType:=Literal;
    Param[2].Value:=IntToStr(getIDrugs);
    if getIDrugs=1 then
    begin
      Param[3].PType:=Literal;
      Param[3].Value:=IntToStr(getWarning);
    end;
    try Call;
    if Results.Count>0 then
    begin
      if Piece(Results[0],'^',1)='1' then
      begin
        displayError('Investigational Drug Use details updated for '+getPatientName+' ('+getPatientLastFour+').',mtInformation);
      end
      else displayError('Broker Error: '+Piece(Results[0],'^',2),mtError);
    end
    else displayError('Broker Error: Failed to update details in VistA',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to update details in VistA',mtError);
    end;
  end;
end;

procedure TdlgManIDrug.btnOKClick(Sender: TObject);
begin
  if Modified then UpdateIDrugs(frmHepC.RPC);
  ClearAllFields;
end;

procedure TdlgManIDrug.cbIDrugsClick(Sender: TObject);
begin
  grpWarning.Visible:=cbIDrugs.Checked;
  isModified;
end;

procedure TdlgManIDrug.FormShow(Sender: TObject);
begin
  setPatientName(edtName);
  setPatientSSN(edtSSN);
  setPatientDOB(edtDOB);
  CallRPCPatientDetails(frmHepC.RPC,getPatientIEN);
  isModified;
end;

procedure TdlgManIDrug.cbWarningClick(Sender: TObject);
begin
  isModified;
end;

procedure TdlgManIDrug.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgManIDrug.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

procedure TdlgManIDrug.FormResize(Sender: TObject);
begin
  pnlMain.Height:=self.ClientHeight-90;
  btnOK.Top:=self.ClientHeight-60;
  btnCancel.Top:=self.ClientHeight-60;
end;

end.
