unit ManPatActive;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Dialogs,
  Buttons, ExtCtrls, ComCtrls, HepCLibrary, MFunStr, TRpcB,
  ORCtrls, MessageBox, HepCCommon, Menus;

type
  TdlgManPatActive = class(TForm)
    btnClose: TButton;
    sbMain: TStatusBar;
    btnActivate: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    pnlMain: TPanel;
    pnlPat: TPanel;
    pnlPatDetails: TPanel;
    edtName: TEdit;
    edtSSN: TEdit;
    edtDOB: TEdit;
    pnlDetails: TPanel;
    grpInfo: TGroupBox;
    edtMessage: TMemo;
    listInfo: TListView;
    grpInactivate: TGroupBox;
    lblReason: TLabel;
    edtReason: TEdit;
    cmbReason: TORComboBox;
    procedure btnActivateClick(Sender: TObject);
    procedure cmbReasonChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    function IsPatientDeceased(DOD2,DOD798:String):Boolean;
    procedure IsPatientActive(InactDate,InactReason,ReactDate,Active:String);
    function getReason:Integer;
    procedure setReason(Reason:Integer);
    procedure setActive(Active:Boolean);
    function getActive:Boolean;
    procedure ClearAllFields;
  public
    { Public declarations }
    function CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
    procedure ActivateInactivate(RPC:TRPCBroker);
  end;

var
  dlgManPatActive: TdlgManPatActive;

implementation

uses PatientSearch, HepC;

{$R *.DFM}

function TdlgManPatActive.IsPatientDeceased(DOD2,DOD798:String):Boolean;
var
  msg:TStrings;
  Today:Integer;
begin
  msg:=TStringList.Create;
  Today:=StrToInt(WindowsDateToFMDate(Date));
  Result:=False;
  if ((DOD2<>'') and (StrToInt(Piece(DOD2,'.',1)) <= Today)) then
  begin
    Result:=True;
    if ((DOD798<>'') and (StrToInt(Piece(DOD798,'.',1)) <= Today)) then
    begin
      msg.Clear;
      msg.Add('Hepatitis C Registry');
      msg.Add('');
      msg.Add('This patient is marked as deceased in the Patient File and the Local Registry.');
      msg.Add('');
      msg.Add('Patients can only be re-activated if NOT marked as deceased in the Patient File.');
      msg.Add('Please contact Decedent Affairs at your facility if the patient''s living status should be changed.');
      ShowMessage(mtInformation,msg);
    end
    else
    begin
      msg.Clear;
      msg.Add('Hepatitis C Registry');
      msg.Add('');
      msg.Add('This patient has a date of death on the Patient File, and yet was NOT marked as deceased in the Hepatitis C Local Registry.');
      msg.Add('A date of death has therefore been added to this patient''s Local Registry entry.');
      msg.Add('');
      msg.Add('Patients can only be re-activated if NOT marked as deceased in the Patient File.');
      msg.Add('Please contact Decedent Affairs at your facility if the patient''s living status should be changed.');
      ShowMessage(mtInformation,msg);
    end;
  end
  else
  begin
    Result:=False;
    if ((DOD798<>'') and (StrToInt(DOD798) <= Today)) then
    begin
      msg.Clear;
      msg.Add('Hepatitis C Registry');
      msg.Add('');
      msg.Add('This patient has no date of death on the Patient File, and yet was marked as deceased in the Hepatitis C Local Registry.');
      msg.Add('The date of death has therefore been removed from this patient''s Local Registry entry.');
      msg.Add('');
      msg.Add('Please contact Decedent Affairs at your facility if the patient''s living status should be changed.');
      ShowMessage(mtInformation,msg);
    end;
  end;
end;

procedure TdlgManPatActive.IsPatientActive(InactDate,InactReason,ReactDate,Active:String);
begin
  setActive(Active='1');
  if Active<>'1' then if InactReason<>'' then setReason(StrToInt(InactReason)) else setReason(0);
end;

function TdlgManPatActive.CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
var
  i:Integer;
begin
  Result:=True;
  ClearAllFields;
  RPC.RemoteProcedure:='ROR ACTIVATE PATIENT GET';
  RPC.Param[0].PType:=Literal;
  RPC.Param[0].Value:=Patien;
  RPC.Param[1].PType:=Literal;
  RPC.Param[1].Value:=RegistryName;
  try RPC.Call;
    if RPC.Results.Count>0 then
    begin
      if IsPatientDeceased(Piece(RPC.Results[1],'^',1),Piece(RPC.Results[1],'^',2)) then
      begin
        ClearAllFields;
        Result:=False;
        Exit;
      end;
      IsPatientActive(Piece(RPC.Results[2],'^',1),Piece(RPC.Results[2],'^',2),Piece(RPC.Results[2],'^',3),Piece(RPC.Results[2],'^',4));
      if RPC.Results.Count>3 then
      begin
        listInfo.Visible:=True;
        edtMessage.Visible:=False;
        for i:=3 to RPC.Results.Count-1 do
        begin
          SetSelectionRule(listInfo,RPC.Results[i]);
        end;
      end
      else
      begin
        listInfo.Visible:=False;
        edtMessage.Visible:=True;
      end;
    end
    else
    begin
      displayError('Broker Error: Failed to retrieve patient data.',mtError);
      Result:=False;
    end;
  except on EBrokerError do
  begin
    displayError('Broker Error: Failed to retrieve patient data.',mtError);
    Result:=False;
  end;
  end;
end;

function TdlgManPatActive.getReason:Integer;
begin
  Result:=cmbReason.ItemIEN;
end;

procedure TdlgManPatActive.setReason(Reason:Integer);
begin
  if Reason>0 then
  begin
    cmbReason.SelectByIEN(Reason);
    edtReason.Text:=cmbReason.Text;
  end
  else edtReason.Text:='No reason given';
  cmbReason.ItemIndex:=-1;
end;

procedure TdlgManPatActive.setActive(Active:Boolean);
begin
  if Active then
  begin
    btnActivate.Caption:='&Inactivate';
    dlgManPatActive.Caption:='Hepatitis C Local Registry - Inactivate Patient';
  end
  else
  begin
    btnActivate.Caption:='&Activate';
    dlgManPatActive.Caption:='Hepatitis C Local Registry - Activate Patient'
  end;
  cmbReason.Visible:=Active;
  edtReason.Visible:=not Active;
  if (Active=False) or ((Active=True) and (getReason>0)) then btnActivate.Enabled:=True;
end;

function TdlgManPatActive.getActive:Boolean;
begin
  Result:=(btnActivate.Caption='&Inactivate');
end;

procedure TdlgManPatActive.ClearAllFields;
begin
  setReason(0);
  listInfo.Items.Clear;
  edtMessage.Visible:=False;
  setActive(False);
  btnActivate.Enabled:=False;
end;

procedure TdlgManPatActive.ActivateInactivate(RPC:TRPCBroker);
var
  msg:TStrings;
begin
  msg:=TStringList.Create;
  msg.Clear;
  if listInfo.Items.Count>0 then
  begin
    if getActive then
    begin
      msg.Clear;
      msg.Add('Hepatitis C Registry');
      msg.Add('');
      msg.Add('The patient you have selected has an ICD-9 code or hepatitis serology test result consistent with Hepatitis C infection.');
      msg.Add('Please be sure that this information was not entered by mistake in your local system files.');
      msg.Add('');
      msg.Add('Continue with inactivation?');
      if ShowMessage(mtConfirmation,msg)=mrNo then
      begin
        msg.Free;
        Exit;
      end;
    end;
  end
  else
  begin
    if not getActive then
    begin
      msg.Clear;
      msg.Add('Hepatitis C Registry');
      msg.Add('');
      msg.Add('You must have confirmatory evidence supporting the history of a positive Hepatitis C antibody test to activate a patient within this Registry.');
      msg.Add('After activation, please make sure that you add the correct ICD-9 code to the patient''s problem list.');
      msg.Add('');
      msg.Add('Continue with activation?');
      if ShowMessage(mtConfirmation,msg)=mrNo then
      begin
        msg.Free;
        Exit;
      end;
    end;
  end;
  with RPC do
  begin
    RemoteProcedure:='ROR ACTIVATE INACTIVATE';
    Param[0].PType:=Literal;
    Param[0].Value:=getPatientIEN;
    Param[1].PType:=Literal;
    Param[1].Value:=RegistryName;
    Param[2].PType:=Literal;
    Param[2].Value:=Copy(btnActivate.Caption,2,1);
    Param[3].PType:=Literal;
    Param[3].Value:=IntToStr(getReason);
    Param[4].PType:=Literal;
    Param[4].Value:=WindowsDateToFMDate(Date)+'.'+WindowsTimeToFMTime(Time);
    try Call;
    if Results.Count>0 then
    begin
      if Piece(Results[0],'^',1)='1' then
      begin
        if getActive then
        begin
          if listInfo.Items.Count>0 then
          begin
            displayError(getPatientName+' ('+getPatientLastFour+') has an ICD-9 code or hepatitis serology test result consistent with Hepatitis C infection but has been '+Piece(Results[0],'^',2)+' from the Hepatitis C registry. Please make sure that the patient has the correct diagnosis in the medical record.',mtWarning);
          end
          else displayError(getPatientName+' ('+getPatientLastFour+') has been '+Piece(Results[0],'^',2)+' from the Hepatitis C registry.',mtInformation);
        end
        else
        begin
          if listInfo.Items.Count>0 then
          begin
            displayError(getPatientName+' ('+getPatientLastFour+') has been ACTIVATED in the Hepatitis C registry.',mtInformation);
          end
          else displayError(getPatientName+' ('+getPatientLastFour+') has been ACTIVATED in the Hepatitis C registry. Please ensure that you add the appropriate ICD-9 code to the Problem List for '+getPatientName+' ('+getPatientLastFour+').',mtWarning);
        end;
      end
      else displayError('Error: '+Piece(Results[0],'^',2),mtError);
    end
    else displayError('Broker Error: Failed to update in VistA',mtError);
    except on EBrokerError do displayError('Broker Error: Failed to update in VistA',mtError);
    end;
  end;
  msg.Free;
end;

procedure TdlgManPatActive.btnActivateClick(Sender: TObject);
begin
  ActivateInactivate(frmHepC.RPC);
  ClearAllFields;
end;

procedure TdlgManPatActive.cmbReasonChange(Sender: TObject);
begin
  if ((getReason>0) or (btnActivate.Caption='Inactivate')) then
  begin
    btnActivate.Enabled:=True;
  end;
end;

procedure TdlgManPatActive.FormShow(Sender: TObject);
begin
  setPatientName(edtName);
  setPatientSSN(edtSSN);
  setPatientDOB(edtDOB);
end;

procedure TdlgManPatActive.FormCreate(Sender: TObject);
begin
  cmbReason.Items.Clear;
  cmbReason.Items.AddStrings(CallRPCGetCodes(frmHepC.RPC,'1'));
end;

procedure TdlgManPatActive.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgManPatActive.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

procedure TdlgManPatActive.FormResize(Sender: TObject);
begin
  pnlMain.Height:=self.ClientHeight-90;
  btnActivate.Top:=self.ClientHeight-60;
  btnClose.Top:=self.ClientHeight-60;
end;

end.
