unit ManPatEdit;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, TRpcB, MFunStr, Dialogs,
  HepCLibrary, HepCCommon, Menus;

type
  TdlgManPatEdit = class(TForm)
    sbMain: TStatusBar;
    btnCancel: TButton;
    btnOK: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    pnlMain: TPanel;
    pnlDetails: TPanel;
    pnlDrugs: TPanel;
    grpDrugs: TGroupBox;
    cbIDrugs: TCheckBox;
    pnlEvidence: TPanel;
    grpEvidence: TRadioGroup;
    pnlRules: TPanel;
    grpInfo: TGroupBox;
    edtMessage: TMemo;
    listInfo: TListView;
    pnlPat: TPanel;
    pnlPatDetails: TPanel;
    edtName: TEdit;
    edtSSN: TEdit;
    edtDOB: TEdit;
    procedure btnOKClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure grpEvidenceClick(Sender: TObject);
    procedure cbIDrugsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    function getEvidence:Integer;
    procedure setEvidence(Evidence:Integer);
    function getIDrugs:Integer;
    procedure setIDrugs(IDrug:Boolean);
    procedure setMessage;
    procedure ClearAllFields;
    procedure isModified;
  public
    { Public declarations }
    function CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
    procedure AddEdit(RPC:TRPCBroker);
  end;

var
  dlgManPatEdit: TdlgManPatEdit;
  NewPatient:Boolean;
  OriginalEvidence: Integer;
  OriginalIDrugs: Boolean;
  Modified: Boolean;
  EvidenceCodes: TStrings;

implementation

uses PatientSearch, HepC;

{$R *.DFM}

procedure TdlgManPatEdit.isModified;
begin
  Modified:=False;
  if OriginalEvidence<>grpEvidence.ItemIndex then Modified:=True;
  if OriginalIDrugs<>cbIDrugs.Checked then Modified:=True;
  if NewPatient then Modified:=True;
  if Modified then sbMain.Panels[0].Text:='Modified' else sbMain.Panels[0].Text:='';
end;

procedure TdlgManPatEdit.ClearAllFields;
begin
  NewPatient:=False;
  setEvidence(0);
  setIDrugs(False);
  listInfo.Items.Clear;
  edtMessage.Visible:=False;
  btnOK.Enabled:=False;
end;

function TdlgManPatEdit.getEvidence:Integer;
begin
  Result:=StrToInt(Piece(EvidenceCodes[grpEvidence.ItemIndex],'^',1));
end;

procedure TdlgManPatEdit.setEvidence(Evidence:Integer);
var
  i:Integer;
begin
  grpEvidence.ItemIndex:=-1;
  for i:=0 to EvidenceCodes.Count-1 do
  begin
    if Piece(EvidenceCodes[i],'^',1)=IntToStr(Evidence) then grpEvidence.ItemIndex:=i;
  end;
  OriginalEvidence:=grpEvidence.ItemIndex;
end;

function TdlgManPatEdit.getIDrugs:Integer;
begin
  if cbIDrugs.Checked then Result:=1 else Result:=0;
end;

procedure TdlgManPatEdit.setIDrugs(IDrug:Boolean);
begin
  cbIDrugs.Checked:=IDrug;
  OriginalIDrugs:=cbIDrugs.Checked;
end;

procedure TdlgManPatEdit.setMessage;
begin
  edtMessage.Visible:=True;
  edtMessage.Lines.Clear;
  edtMessage.Lines.Add('No information regarding Hepatitis C testing found in this patient''s profile.');
  if NewPatient then edtMessage.Lines.Add('You must have confirmatory evidence supporting a positive Hepatitis C antibody test to add a patient to the Registry. When you have finished with this process, please make sure that you add the correct ICD-9 code to this patient''s problem list.');
  listInfo.Visible:=False;
end;

function TdlgManPatEdit.CallRPCPatientDetails(RPC:TRPCBroker;PatIEN:String):Boolean;
var
  i:Integer;
begin
  Result:=True;
  ClearAllFields;
  RPC.RemoteProcedure:='ROR EDIT PATIENT GET';
  RPC.Param[0].PType:=Literal;
  RPC.Param[0].Value:=Patien;
  RPC.Param[1].PType:=Literal;
  RPC.Param[1].Value:=RegistryName;
  try RPC.Call;
    if RPC.Results.Count>0 then
    begin
      NewPatient:=(RPC.Results[0]='0');
      if NewPatient then
      begin
        dlgManPatEdit.Caption:='Hepatitis C Local Registry - Add Patient';
        btnOK.Caption:='&Add';
      end
      else
      begin
        dlgManPatEdit.Caption:='Hepatitis C Local Registry - Edit Patient';
        btnOK.Caption:='&OK';
      end;
      setEvidence(StrToInt(RPC.Results[1]));
      if RPC.Results[2]='1' then setIDrugs(True) else setIDrugs(False);
      if RPC.Results.Count>3 then
      begin
        listInfo.Visible:=True;
        for i:=3 to RPC.Results.Count-1 do
        begin
          setSelectionRule(listInfo,RPC.Results[i]);
        end;
      end
      else setMessage;
    end
    else
    begin
      displayError('Broker Error: Failed to retrieve patient details from VistA',mtError);
      Result:=False;
    end;
  except on EBrokerError do
  begin
    displayError('Broker Error: Failed to retrieve patient details from VistA',mtError);
    Result:=False;
  end;
  end;
end;

procedure TdlgManPatEdit.AddEdit(RPC:TRPCBroker);
var
  EnterUpdate:String;
begin
  if NewPatient then EnterUpdate:='entered' else EnterUpdate:='updated';
  with RPC do
  begin
    RemoteProcedure:='ROR ENTER EDIT PATIENT';
    Param[0].PType:=Literal;
    Param[0].Value:=GetPatientIEN;
    Param[1].PType:=Literal;
    Param[1].Value:=RegistryName;
    Param[2].PType:=Literal;
    Param[2].Value:=IntToStr(GetEvidence);
    Param[3].PType:=Literal;
    Param[3].Value:=IntToStr(GetIDrugs);
    Param[4].PType:=List;
    Param[4].Mult.Assign(GetSelectionRules(listInfo));
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='1' then
        begin
          displayError(getPatientName+' ('+getPatientLastFour+') '+EnterUpdate+' in Local Registry',mtInformation);
        end
        else displayError('Update Failed: '+Piece(Results[0],'^',2),mtError);
      end
      else displayError('Broker Error: There was an error filing this data',mtError);
    except on EBrokerError do displayError('Broker Error: There was an error filing this data',mtError);
    end;
  end;
end;

procedure TdlgManPatEdit.btnOKClick(Sender: TObject);
begin
  if Modified then AddEdit(frmHepC.RPC);
  self.Close;
end;

procedure TdlgManPatEdit.FormShow(Sender: TObject);
begin
  setPatientName(edtName);
  setPatientSSN(edtSSN);
  setPatientDOB(edtDOB);
  isModified;
end;

procedure TdlgManPatEdit.grpEvidenceClick(Sender: TObject);
begin
  btnOK.Enabled:=(grpEvidence.ItemIndex>-1);
  isModified;
end;

procedure TdlgManPatEdit.cbIDrugsClick(Sender: TObject);
begin
  isModified;
end;

procedure TdlgManPatEdit.FormCreate(Sender: TObject);
var
  i:Integer;
begin
  EvidenceCodes:=TStringList.Create;
  EvidenceCodes.Clear;
  EvidenceCodes.AddStrings(CallRPCGetCodes(frmHepC.RPC,'2'));
  grpEvidence.Items.Clear;
  for i:=0 to EvidenceCodes.Count-1 do
  begin
    grpEvidence.Items.Add(Piece(EvidenceCodes[i],'^',2));
  end;
  pnlEvidence.Height:=(22 * grpEvidence.Items.Count);
end;

procedure TdlgManPatEdit.FormDestroy(Sender: TObject);
begin
  EvidenceCodes.Free;
end;

procedure TdlgManPatEdit.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgManPatEdit.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

procedure TdlgManPatEdit.FormResize(Sender: TObject);
begin
  pnlMain.Height:=self.ClientHeight-90;
  btnOK.Top:=self.ClientHeight-60;
  btnCancel.Top:=self.ClientHeight-60;
end;

end.
