unit PtSelect;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  TRpcB,
  HepCLibrary, Menus;

function CallServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; RetList: TStringList = nil): Boolean;
function PatientSelect(Broker: TRPCBroker; DFN: string): Boolean;

type
  TMessageLevel = (mlError, mlInfo, mlWarning, mlSecurity);

type
  TPatient = class(TObject)
  private
    FDFN: string;
    FIdentifiers: TList;
    FMessages: TList;
    FSensitive: Boolean;
    procedure SetSensitive(NewVal: Boolean);
  public
    constructor Create;
    destructor Destroy; override;
  published
    property DFN: string
      read FDFN write FDFN;
    property Identifiers: TList
      read FIdentifiers write FIdentifiers;
    property Messages: TList
      read FMessages write FMessages;
    property Sensitive: Boolean
      read FSensitive write SetSensitive;
  end;

type
  TPtIdentifier = class(TObject)
  private
    FCaption: string;
    FDisplayNonSensitive: string;
    FDisplaySensitive: string;
    FDisplayLabel: TEdit;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Caption: string
      read FCaption write FCaption;
    property DisplayNonSensitive: string
      read FDisplayNonSensitive write FDisplayNonSensitive;
    property DisplaySensitive: string
      read FDisplaySensitive write FDisplaySensitive;
    property DisplayLabel: TEdit
      read FDisplayLabel write FDisplayLabel;
  end;

type
  TPtMessage = class(TObject)
  protected
    FLevel: TMessageLevel;
    FHeader: string;
    FText: TStringList;
    FPanel: TPanel;
  public
    constructor Create(Level: TMessageLevel; HeaderText: string);
    destructor Destroy; override;
    property MessageLevel: TMessageLevel
      read FLevel write FLevel;
    property Header: string
      read FHeader write FHeader;
    property Text: TStringList
      read FText write FText;
    property Panel: TPanel
      read FPanel write FPanel;
  end;

type
  TfrmPtSelect = class(TForm)
    btnCancel: TButton;
    btnOk: TButton;
    pnlPtInfo: TPanel;
    btnAgree: TButton;
    procedure btnAgreeClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
  private
    procedure LoadPatient(Patient: TPatient);
    procedure LoadMessages(Patient: TPatient);
  end;

var
  frmPtSelect: TfrmPtSelect;
  CurrentPatient: TPatient;

const
  RPC_LOGACCESS = 'ROR PATIENT CHECKS';

implementation

uses MFunStr, HepC;

{$R *.DFM}

function CallServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; RetList: TStringList = nil): Boolean;
var
  i: integer;
begin
  Broker.RemoteProcedure := RemoteProcedure;
  i := 0;
  while i <= High(Parameters) do
    if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
      begin
        Broker.Param[i].Value := Copy(Parameters[i], 2, Length(Parameters[i]));
        Broker.Param[i].PType := Reference;
        inc(i);
      end
    else
      begin
        Broker.Param[i].Value := Parameters[i];
        Broker.Param[i].PType := Literal;
        inc(i);
      end;

  if MultList <> nil then
    begin
      Broker.Param[i + 1].Mult.Assign(MultList);
      Broker.Param[i + 1].PType := List;
    end;

  try
    if RetList = nil then
      Broker.Call
    else
      Broker.lstCall(RetList);
    Result := True;
  except
    on E: Exception do
      begin
        displayError('Broker Error: '+E.Message,mtError);
        //MessageDlg(
        //  'Error encountered retrieving VistA data.' + #13 +
        //  'Server: ' + Broker.Server + #13 +
        //  'Listener port: ' + IntToStr(Broker.ListenerPort) + #13 +
        //  'Remote procedure: ' + Broker.RemoteProcedure + #13 +
        //  'Error is: ' + #13 +
        //  E.Message, mtError, [mbok], E.HelpContext);
        Result := False;
        Broker.Results.Insert(0, '-1^Broker Error');
      end;
  end;

  Screen.Cursor := crDefault;
end;

function PatientSelect(Broker: TRPCBroker; DFN: string): Boolean;
var
  i: integer;
  Patient: TPatient;
  PtID: TPtIdentifier;
  PtMsg: TPtMessage;
  PtResults: TStringList;
begin
  {Create Patient Object}
  Patient := TPatient.Create;
  Patient.DFN := DFN;
  Patient.Sensitive := False;
  CurrentPatient := Patient;

  with Broker do
    begin
      RemoteProcedure := RPC_LOGACCESS;
      Param[0].Value := 'SELECT';
      Param[0].PType := Literal;
      Param[1].Value := DFN;
      Param[1].PType := Literal;
      Call;
      PtResults := TStringList.Create;
      PtResults.Assign(Broker.Results);
    end;

  try
    {Build the Patient Identifiers}
    i := 0;
    while i < PtResults.Count do
      begin
        if piece(PtResults[i], '^', 1) = '$$PTID' then
          begin
            PtId := TPtIdentifier.Create;
            PtId.Caption := piece(PtResults[i], '^', 2);
            PtId.DisplayNonSensitive := piece(PtResults[i], '^', 3);
            PtId.DisplaySensitive := piece(PtResults[i], '^', 4);
            if PtId.DisplaySensitive = '' then
              PtId.DisplaySensitive := PtId.DisplayNonSensitive;
            Patient.Identifiers.Add(ptId);
          end;
        inc(i);
      end;

    {Load any messages}
    i := 0;
    while i < PtResults.Count - 1 do
      begin
        if piece(PtResults[i], '^', 1) = '$$MSGHDR' then
          begin
            case StrToIntDef(piece(PtResults[i], '^', 2), -1) of
              0: PtMsg := TPtMessage.Create(mlError, piece(PtResults[i], '^', 3));
              1: PtMsg := TPtMessage.Create(mlInfo, piece(PtResults[i], '^', 3));
              2: PtMsg := TPtMessage.Create(mlWarning, piece(PtResults[i], '^', 3));
              3:
                begin
                  PtMsg := TPtMessage.Create(mlSecurity, piece(PtResults[i], '^', 3));
                  Patient.Sensitive := True;
                end;
            else
              PtMsg := TPtMessage.Create(mlInfo, piece(PtResults[i], '^', 3));
            end;

            inc(i);

            while (i < PtResults.Count - 1) and
              (piece(PtResults[i], '^', 1) <> '$$MSGEND') do
              begin
                ptMsg.Text.Add(PtResults[i]);
                inc(i);
              end;
            Patient.Messages.Add(PtMsg);
          end;
        inc(i);
      end;

    with TfrmPtSelect.Create(Application) do
      begin
        LoadPatient(Patient);
        LoadMessages(Patient);
        Position := poDesktopCenter;
        ShowModal;
        if ModalResult = mrOk then
          PatientSelect := True
        else
          PatientSelect := False;
      end;
  finally
    Patient.Free;
  end;

end;

//* TPtMessage *//

constructor TPtMessage.Create(Level: TMessageLevel; HeaderText: string);
begin
  FText := TStringList.Create;
  FLevel := Level;
  FHeader := HeaderText;
end;

destructor TPtMessage.Destroy;
begin
  FText.Clear;
  FText.Free;
end;

//* TPtIdentifier *//

constructor TPtIdentifier.Create;
begin
  inherited Create;
end;

destructor TPtIdentifier.Destroy;
begin
  inherited Destroy;
end;

//* TPatient *//

constructor TPatient.Create;
begin
  inherited Create;
  FIdentifiers := TList.Create;
  FMessages := TList.Create;
end;

destructor TPatient.Destroy;
begin

  while FIdentifiers.Count > 0 do
    begin
      TPtIdentifier(FIdentifiers[0]).Free;
      FIdentifiers.Delete(0);
    end;
  FIdentifiers.Free;

  while FMessages.Count > 0 do
    begin
      TPtMessage(FMessages[0]).Free;
      FMessages.Delete(0);
    end;
  FMessages.Free;

  inherited Destroy;

end;

procedure TPatient.SetSensitive(NewVal: Boolean);
var
  i: integer;
begin
  FSensitive := NewVal;
  for i := 0 to Self.FIdentifiers.Count - 1 do
    with TPtIdentifier(Self.FIdentifiers[i]) do
      case NewVal of
        True: DisplayLabel.Text := DisplaySensitive;
        False: DisplayLabel.Text := DisplayNonSensitive;
      end;
end;

//* TfrmPtSelect *//

procedure TfrmPtSelect.LoadPatient(Patient: TPatient);
var
  i: integer;
  PtId: TPtIdentifier;
begin
  for i := 0 to Patient.Identifiers.Count - 1 do
    begin

      PtId := TPtIdentifier(Patient.Identifiers[i]);

      with TLabel.Create(pnlPtInfo) do
        begin
          Parent := pnlPtInfo;
          AutoSize := False;
          Left := 15;
          Width := 155;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          Caption := PtId.Caption + ':';
        end;
      PtId.DisplayLabel := TEdit.Create(pnlPtInfo);

      with PtId.DisplayLabel do
        begin
          Parent := pnlPtInfo;
          Left := 170;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          //Font.Style := [fsBold];
          BorderStyle := bsNone;
          ReadOnly := True;
          AutoSelect := False;
          ParentColor := True;
          Height := 14;
          TabStop := True;
          if Patient.Sensitive then
            Text := PtId.DisplaySensitive
          else
            Text := PtId.DisplayNonSensitive;
        end;

    end;

  pnlPtInfo.Height := Patient.Identifiers.Count * (Self.Canvas.TextHeight('X') + 8);

  Height := pnlPtInfo.Height + 70;
  Resize;
end;

procedure TfrmPtSelect.LoadMessages(Patient: TPatient);
var
  ErrorReturned: Boolean;
  msgBitMap: TBitMap;
  PtMsg: TPtMessage;
  i: integer;
begin
  ErrorReturned := False;
  msgBitMap := TBitMap.Create;
  msgBitMap.Transparent := True;
  msgBitMap.TransparentMode := tmAuto;

  for i := 0 to Patient.Messages.Count - 1 do
    begin
      PtMsg := TPtMessage(Patient.Messages[i]);

      PtMsg.Panel := TPanel.Create(Self);

      with PtMsg.Panel do
        begin
          Caption := '';
          BevelInner := bvNone;
          BevelOuter := bvNone;
          Parent := Self;

          if i > 0 then
            begin
              Top := TPtMessage(Patient.Messages[i - 1]).Panel.Top +
                TPtMessage(Patient.Messages[i - 1]).Panel.Height;
            end
          else
            Top := pnlPtInfo.Top + pnlPtInfo.Height + 5;

          Left := 5;
          Width := Self.Width - (self.BorderWidth * 2) - 5;
          Height := PtMsg.Text.Count * (Canvas.TextHeight('X') + 4);
          if Height < 40 then
            Height := 40;
        end;

      with TLabel.Create(PtMsg.Panel) do
        begin
          Parent := PtMsg.Panel;
          AutoSize := False;
          Top := 5;
          Left := 45;
          Width := PtMsg.Panel.Width - 5;
          WordWrap := True;
          Caption := PtMsg.Header;
          Font.Style := [fsBold];
          AutoSize := True;
        end;

      with TLabel.Create(PtMsg.Panel) do
        begin
          Parent := PtMsg.Panel;
          AutoSize := False;
          Top := 10 + Self.Canvas.TextHeight('X');
          Left := 45;
          Width := PtMsg.Panel.Width - 50;
          WordWrap := True;
          Caption := PtMsg.Text.Text;
          AutoSize := True;
          PtMsg.Panel.Height := Height + 15;
          self.Height := PtMsg.Panel.Top + PtMsg.Panel.Height + 70;
          self.Resize;
        end;

      with TImage.Create(Self) do
        begin
          Transparent := True;
          Parent := PtMsg.Panel;
          Left := 5;
          Top := 5;
          Height := 32;
          Width := 32;
          case PtMsg.MessageLevel of
            mlInfo: msgBitMap.LoadFromResourceName(HInstance, 'INFO');
            mlWarning: msgBitmap.LoadFromResourceName(HInstance, 'WARNING');
            mlSecurity:
              begin
                msgBitmap.LoadFromResourceName(HInstance, 'SECURITY');
                btnOk.Enabled := False;
                btnOk.TabStop := False;
                btnOk.Default := False;
                btnAgree.Visible := True;
                btnAgree.TabStop := True;
              end;
            mlError:
              begin
                msgBitmap.LoadFromResourceName(HInstance, 'ERROR');
                btnOk.ModalResult := mrCancel;
                btnCancel.Visible := False;
                ErrorReturned := True;
              end;
          end;
          Picture.Bitmap := msgBitMap;
        end;
    end;

  if ErrorReturned then
    begin
      btnOk.ModalResult := mrCancel;
      btnOk.Visible := True;
      btnCancel.Visible := False;
      btnAgree.Visible := False;
    end;

  Caption := 'Selected Patient';
  if (Patient.Messages.Count > 0) and (ErrorReturned = False) then
    Caption := 'Selected Patient - Please read notifications carefully';
  if (Patient.Messages.Count > 0) and (ErrorReturned = True) then
    Caption := 'Patient Selection - Unable to select this record';

end;

procedure TfrmPtSelect.btnAgreeClick(Sender: TObject);
begin
  CallServer(frmHepC.RPC, RPC_LOGACCESS, ['LOGSECURITY', CurrentPatient.DFN, 'RPCCALL^Hepatitis C Registry GUI v1'], nil);
  btnOk.Enabled := True;
  btnOk.Default := True;
  btnOk.TabStop := True;
  btnAgree.Enabled := False;
  btnAgree.Default := False;
  btnAgree.TabStop := False;
  btnAgree.Caption := 'Access has been logged';
  CurrentPatient.Sensitive := False;
  btnOk.SetFocus;
end;

procedure TfrmPtSelect.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TfrmPtSelect.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

end.
