unit Report;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, Menus, ComCtrls, LocRegPatRep, HepCLibrary,
  MFunStr, HepCCommon, Dialogs;

type
  TdlgReport = class(TForm)
    btnOK: TButton;
    StatusBar1: TStatusBar;
    btnPrint: TButton;
    pnlMain: TPanel;
    pnlDetails: TPanel;
    pnlReport: TPanel;
    listReport: TListView;
    PrintDialog1: TPrintDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    HepCRegistryHelp1: TMenuItem;
    pnlRepDets: TPanel;
    lblSortedBy: TLabel;
    edtSortedBy: TEdit;
    lblStartDate: TLabel;
    edtStartDate: TEdit;
    lblEndDate: TLabel;
    edtEndDate: TEdit;
    lblActive: TLabel;
    edtReport: TEdit;
    pnlExtDets: TPanel;
    lblCreated: TLabel;
    edtCreated: TEdit;
    lblUpdated: TLabel;
    edtUpdated: TEdit;
    lblExtracted: TLabel;
    edtExtracted: TEdit;
    procedure btnPrintClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure HepCRegistryHelp1Click(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    procedure setCreated;
    procedure setUpdated;
    procedure setExtracted;
  public
    { Public declarations }
    procedure setSortedBy(SortedBy:String);
    procedure setStartDate(StartDate:String);
    procedure setEndDate(EndDate:String);
    procedure setReport(ReportType:String);
    procedure setReportItem(Data,AddFlds:String);
    procedure CreateColumns(AddFlds:String);
  end;

var
  dlgReport: TdlgReport;

implementation

{$R *.DFM}

procedure TdlgReport.setSortedBy(SortedBy:String);
begin
  edtSortedBy.Text:=SortedBy;
end;

procedure TdlgReport.setStartDate(StartDate:String);
begin
  if StartDate<>'' then edtStartDate.Text:=DateToStr(FMDateToWindowsDate(StartDate)) else edtStartDate.Text:='None';
end;

procedure TdlgReport.setEndDate(EndDate:String);
begin
  if EndDate<>'' then edtEndDate.Text:=DateToStr(FMDateToWindowsDate(EndDate)) else edtEndDate.Text:='None';
end;

procedure TdlgReport.setReport(ReportType:String);
begin
  if ReportType='B' then edtReport.Text:='Active & Inactive patients'
  else if ReportType='I' then edtReport.Text:='Inactive patients'
  else edtReport.Text:='Active patients';
end;

procedure TdlgReport.setCreated;
begin
  edtCreated.Text:=DateToStr(Date);
end;

procedure TdlgReport.setUpdated;
begin
  edtUpdated.Text:=LastUpdate;
end;

procedure TdlgReport.setExtracted;
begin
  edtExtracted.Text:=LastExtract;
end;

procedure TdlgReport.btnPrintClick(Sender: TObject);
begin
  if PrintDialog1.Execute then PrintReport;
end;

procedure TdlgReport.Exit1Click(Sender: TObject);
begin
  Application.Terminate;
  Exit;
end;

procedure TdlgReport.setReportItem(Data,AddFlds:String);
var
  i:Integer;
  Item:TListItem;
begin
  Item:=listReport.Items.Add;
  Item.Caption:=Piece(Data,'^',1);
  Item.SubItems.Add(Piece(Data,'^',2));
  Item.SubItems.Add(Piece(Data,'^',3));
  for i:=1 to 9 do
  begin
    if Piece(AddFlds,'^',i)='1' then
    begin
      Item.SubItems.Add(Piece(Data,'^',3+i));
    end;
  end;
end;

procedure TdlgReport.CreateColumns(AddFlds:String);
var
  i:Integer;
  Col:TListColumn;
begin
  while listReport.Columns.Count>3 do listReport.Columns.Delete(listReport.Columns.Count-1);
  for i:=1 to 9 do
  begin
    if Piece(AddFlds,'^',i)='1' then
    begin
      Col:=listReport.Columns.Add;
      case i of
      1: Col.Caption:='Patient DFN';
      2: Col.Caption:='Patient ICN';
      3: Col.Caption:='Date of Birth';
      4: Col.Caption:='Entered';
      5: Col.Caption:='Inactivated';
      6: Col.Caption:='Reason Inact';
      7: Col.Caption:='Reactivated';
      8: Col.Caption:='Inv Drugs';
      9: Col.Caption:='Reason Added';
      end;
      if i=9 then Col.Width:=160 else if i=6 then Col.Width:=100 else Col.Width:=80;
    end;
  end;
end;

procedure TdlgReport.FormShow(Sender: TObject);
begin
  setCreated;
  setUpdated;
  setExtracted;
end;

procedure TdlgReport.HepCRegistryHelp1Click(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER,0);
end;

procedure TdlgReport.FormResize(Sender: TObject);
begin
  pnlMain.Height:=self.ClientHeight-90;
  btnOK.Top:=self.ClientHeight-60;
  btnPrint.Top:=self.ClientHeight-60;
end;

end.
