unit dlgDrugActivityDates;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, HepCLibrary, Dialogs, Printers, TRpcB,
  FileCtrl, MFunStr;

type
  TRPCHandle = class(TObject)
  private
    FID:String;
    FTaskStatus:String;
  public
    constructor Create;
    destructor Destroy;override;
  published
    property ID:String
      read FID write FID;
    property TaskStatus:String
      read FTaskStatus write FTaskStatus;
  end;

type
  TdlgDrugDates = class(TForm)
    StatusBar1: TStatusBar;
    PrintDialog1: TPrintDialog;
    SaveDialog1: TSaveDialog;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    pnlMain: TPanel;
    pnlDates: TPanel;
    grpDates: TGroupBox;
    lblStartDate: TLabel;
    lblEndDate: TLabel;
    dtpStartDate: TDateTimePicker;
    dtpEndDate: TDateTimePicker;
    cbStartDate: TCheckBox;
    cbEndDate: TCheckBox;
    pnlTimePeriod: TPanel;
    grpTimePeriod: TRadioGroup;
    cmbQuarterly: TComboBox;
    pnlYear: TPanel;
    LeftLine: TBevel;
    lblYear: TLabel;
    cmbYear: TComboBox;
    pnlDateRange: TPanel;
    grpDateRange: TRadioGroup;
    btnOK: TButton;
    TabSheet3: TTabSheet;
    pnlOptions: TPanel;
    pnlButtons: TPanel;
    btnExport: TSpeedButton;
    btnShow: TSpeedButton;
    btnPrint: TSpeedButton;
    btnCancel: TButton;
    procedure grpDateRangeClick(Sender: TObject);
    procedure cbStartDateClick(Sender: TObject);
    procedure cbEndDateClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnShowClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure displayError(ErrMessage:String);
    function getStartDate:String;
    procedure setStartDate(StartDate:String);
    function getEndDate:String;
    procedure setEndDate(EndDate:String);
    function getDateRange:Integer;
    procedure setDateRange(DateRange:Integer);
    procedure ReportLineToCanvas(var Line:Integer;DataString:String;Bold:Boolean);
    procedure printHeader(var Line:Integer);
    function CallRPCReportText:TStrings;
    procedure CallRPCGetHandle(var Hndl:TRPCHandle);
    procedure CallRPCTaskReport(StartDate,EndDate:String);
    procedure CallRPCStoreHandle;
    procedure ExportReport(FileName:String);
    procedure DisplayReport;
  public
    { Public declarations }
    procedure PrintReport;
    procedure getStartEndDates(var StartDate:String;var EndDate:String);
  end;

var
  dlgDrugDates: TdlgDrugDates;
  RPCHandle:TRPCHandle;

implementation

uses HepC, DrugReport;

{$R *.DFM}

constructor TRPCHandle.Create;
begin
  inherited Create;
  FID:='';
  FTaskStatus:='BadHandle';
end;

destructor TRPCHandle.Destroy;
begin
  inherited Destroy;
end;

procedure TdlgDrugDates.displayError(ErrMessage:String);
begin
  MessageDlg('Hepatitis C Registry'+#13#13+ErrMessage,mtError,[mbOK],0);
end;

function TdlgDrugDates.getStartDate:String;
begin
  if cbStartDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpStartDate.Date);
end;

procedure TdlgDrugDates.setStartDate(StartDate:String);
begin
  if StartDate<>'' then dtpStartDate.Date:=FMDateToWindowsDate(StartDate) else
  begin
    dtpStartDate.Date:=StrToDate(LastExtract);
    cbStartDate.Checked:=True;
  end;
end;

function TdlgDrugDates.getEndDate:String;
begin
  if cbEndDate.Checked then Result:='' else Result:=WindowsDateToFMDate(dtpEndDate.Date);
end;

procedure TdlgDrugDates.setEndDate(EndDate:String);
begin
  if EndDate<>'' then dtpEndDate.Date:=FMDateToWindowsDate(EndDate) else
  begin
    dtpEndDate.Date:=Date;
    cbEndDate.Checked:=True;
  end;
end;

procedure TdlgDrugDates.getStartEndDates(var StartDate:String;var EndDate:String);
var
  Year:Integer;
begin
  if grpDateRange.ItemIndex=0 then
  begin
    StartDate:=GetStartDate;
    EndDate:=GetEndDate;
  end
  else
  begin
    Year:=StrToInt(cmbYear.Items[cmbYear.ItemIndex]);
    case grpTimePeriod.ItemIndex of
    0:
      begin
        StartDate:=ConstructDate(Year,1,1);
        EndDate:=ConstructDate(Year,12,31);
      end;
    1:
      begin
        StartDate:=ConstructDate(Year-1,10,1);
        EndDate:=ConstructDate(Year,9,30);
      end;
    2:
      begin
        case cmbQuarterly.ItemIndex of
        0:
          begin
            StartDate:=ConstructDate(Year,1,1);
            EndDate:=ConstructDate(Year,3,31);
          end;
        1:
          begin
            StartDate:=ConstructDate(Year,4,1);
            EndDate:=ConstructDate(Year,6,30);
          end;
        2:
          begin
            StartDate:=ConstructDate(Year,7,1);
            EndDate:=ConstructDate(Year,9,30);
          end;
        3:
          begin
            StartDate:=ConstructDate(Year,10,1);
            EndDate:=ConstructDate(Year,12,31);
          end;
        end;
      end;
    end;
  end;
end;

function TdlgDrugDates.getDateRange:Integer;
begin
  Result:=grpDateRange.ItemIndex;
end;

procedure TdlgDrugDates.setDateRange(DateRange:Integer);
begin
  grpDateRange.ItemIndex:=DateRange;
end;

procedure TdlgDrugDates.grpDateRangeClick(Sender: TObject);
begin
  if grpDateRange.ItemIndex=0 then
  begin
    pnlTimePeriod.Visible:=False;
    pnlDates.Top:=37;
    pnlDates.Visible:=True;
  end
  else
  begin
    pnlDates.Visible:=False;
    pnlDates.Top:=37;
    pnlTimePeriod.Visible:=True;
  end;
end;

procedure TdlgDrugDates.cbStartDateClick(Sender: TObject);
begin
  dtpStartDate.Enabled:=(not cbStartDate.Checked);
end;

procedure TdlgDrugDates.cbEndDateClick(Sender: TObject);
begin
  dtpEndDate.Enabled:=(not cbEndDate.Checked);
end;

procedure TdlgDrugDates.ReportLineToCanvas(var Line:Integer;DataString:String;Bold:Boolean);
var
  LeftPos,ColWidth,i:Integer;
begin
  //put blank line in before each patient
  if Piece(DataString,'^',1)<>'' then
  begin
    Line:=Line+1;
    if ((80*Line)>Printer.PageHeight) then
    begin
      Printer.NewPage;
      printHeader(Line);
    end;
  end;
  if Bold then Printer.Canvas.Font.Style:=[fsBold];
  Printer.Canvas.TextOut(0,80*(Line-1),Piece(DataString,'^',1));
  Printer.Canvas.TextOut(1360,80*(Line-1),Piece(DataString,'^',2));
  Printer.Canvas.TextOut(1680,80*(Line-1),Piece(DataString,'^',3));
  Printer.Canvas.TextOut(3240,80*(Line-1),Piece(DataString,'^',4));
  Printer.Canvas.TextOut(3760,80*(Line-1),Piece(DataString,'^',5));
  if Bold then Printer.Canvas.Font.Style:=[];
  Line:=Line+1;
  if ((80*Line)>Printer.PageHeight) then
  begin
    Printer.NewPage;
    printHeader(Line);
  end;
end;

procedure TdlgDrugDates.printHeader(var Line:Integer);
var
  StartDate,EndDate:String;
  str:String;
  chrw,chrh:Integer;
begin
  GetStartEndDates(StartDate,EndDate);
  //
  str:='Hepatitis C - Drug Therapy List';
  chrw:=Printer.Canvas.TextExtent(str).cx;
  chrh:=Printer.Canvas.TextExtent(str).cy;
  Printer.Canvas.Font.Style:=[fsBold,fsUnderline];
  Printer.Canvas.TextOut((Printer.PageWidth-chrw) div 2,0,str);
  Printer.Canvas.Font.Style:=[];
  str:='Page: '+IntToStr(Printer.PageNumber);
  chrw:=Printer.Canvas.TextExtent(str).cx;
  Printer.Canvas.TextOut(Printer.PageWidth-chrw,0,str);
  //
  if Printer.PageNumber=1 then
  begin
    str:='Start Date: ';
    if StartDate<>'' then str:=str+DateToStr(FMDateToWindowsDate(StartDate)) else str:=str+' None';
    Printer.Canvas.TextOut(0,chrh*2,str);
    str:='Report Created: '+DateToStr(Date);
    chrw:=Printer.Canvas.TextExtent(str).cx;
    Printer.Canvas.TextOut(Printer.PageWidth-chrw,chrh*2,str);
    //
    str:='End Date: ';
    if EndDate<>'' then str:=str+DateToStr(FMDateToWindowsDate(EndDate)) else str:=str+' None';
    Printer.Canvas.TextOut(0,chrh*3,str);
    //
    Line:=5;
  end
  else Line:=2;
  ReportLineToCanvas(Line,'NAME^LAST^DRUG^NDC^FILL',True);
  ReportLineToCanvas(Line,'^FOUR^NAME^^DATE',True);
end;

procedure TdlgDrugDates.CallRPCTaskReport(StartDate,EndDate:String);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='XWB DEFERRED RPC';
    Param[0].PType:=Literal;
    Param[0].Value:='ROR DRUG REPORT';
    Param[1].PType:=Literal;
    Param[1].Value:=RegistryName;
    Param[2].PType:=Literal;
    Param[2].Value:=StartDate;
    Param[3].PType:=Literal;
    Param[3].Value:=EndDate;
    try Call;
    if Results.Count>0 then
    begin
      if Piece(Results[0],'^',1)<>'-1' then
      begin
        RPCHandle.ID:=Results[0];
        RPCHandle.TaskStatus:='New';
      end
      else displayError('Broker Error: '+Piece(Results[0],'^',2));
    end
    else displayError('Broker Error: Failed to schedule report');
    except on EBrokerError do displayError('Broker Error: Failed to schedule report');
    end;
    if RPCHandle.ID<>'' then CallRPCStoreHandle;
  end;
end;

procedure TdlgDrugDates.CallRPCGetHandle(var Hndl:TRPCHandle);
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR GET RPC HANDLE';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)<>'-1' then
        begin
          Hndl.ID:=Results[0];
          Hndl.TaskStatus:=Results[1];
        end
        else displayError('Broker Error:'+Piece(Results[0],'^',2));
      end
      else displayError('Broker Error: Failed to retrieve Handle');
    except on EBrokerError do displayError('Broker Error: Failed to retrieve Handle');
    end;
  end;
end;

procedure TdlgDrugDates.CallRPCStoreHandle;
begin
  with frmHepC.RPC do
  begin
    RemoteProcedure:='ROR STORE RPC HANDLE';
    Param[0].PType:=Literal;
    Param[0].Value:=RegistryName;
    Param[1].PType:=Literal;
    Param[1].Value:=RPCHandle.ID;
    try Call;
      if Results.Count>0 then
      begin
        if Piece(Results[0],'^',1)='-1' then displayError('Broker Error: '+Piece(Results[0],'^',2));
      end
      else displayError('Broker Error: Failed to store Handle');
    except on EBrokerError do displayError('Broker Error: Failed to store Handle');
    end;
  end;
end;

function TdlgDrugDates.CallRPCReportText:TStrings;
begin
  Result:=TStringList.Create;
  Result.Clear;
  CallRPCGetHandle(RPCHandle);
  if ((RPCHandle.ID<>'') and (RPCHandle.TaskStatus='Done')) then
  begin
    with frmHepC.RPC do
    begin
      RemoteProcedure:='XWB DEFERRED GETDATA';
      Param[0].PType:=Literal;
      Param[0].Value:=RPCHandle.ID;
      try Call;
        if Results.Count>0 then
        begin
          Result.AddStrings(Results);
        end
        else Result.Add('Nothing to report.');
      except on EBrokerError do displayError('Broker Error: Failed to retrieve report info');
      end;
    end;
  end
  else if ((RPCHandle.ID<>'') and (RPCHandle.TaskStatus='Running')) then displayError('Previously scheduled report still running')
  else displayError('Cannot find previous report');
end;

procedure TdlgDrugDates.PrintReport;
var
  Index,Line:Integer;
begin
  Printer.Title:='Hepatitis C Drug Therapy List';
  Printer.Canvas.Font.Size:=8;
  Printer.BeginDoc;
  printHeader(Line);
  for Index:=0 to DrugReportData.Count-1 do
  begin
    ReportLineToCanvas(Line,DrugReportData[Index],False);
  end;
  Printer.EndDoc;
  MessageDlg('Hepatitis C Local Registry'+#13#13+Printer.Title+' sent to printer.',mtInformation,[mbOK],0);
end;

procedure TdlgDrugDates.ExportReport(FileName:String);
var
  SaveText:TextFile;
  str:String;
  Index,i:Integer;
  ReportText:TStrings;
begin
  ReportText:=TStringList.Create;
  ReportText.Clear;
  DrugReportData:=CallRPCReportText;
  //
  ReportText.Add('NAME'+#9+'LAST FOUR'+#9+'DRUG NAME'+#9+'NDC'+#9+'FILL DATE');
  for Index:=0 to DrugReportData.Count-1 do
  begin
    str:='';
    for i:=1 to 4 do str:=str+Piece(DrugReportData[Index],'^',i)+#9;
    str:=str+Piece(DrugReportData[Index],'^',5);
    ReportText.Add(str);
  end;
  AssignFile(SaveText,FileName);
  Rewrite(SaveText);
  Write(SaveText,ReportText.Text);
  CloseFile(SaveText);
  //
  ReportText.Destroy;
end;

procedure TdlgDrugDates.DisplayReport;
var
  Index:Integer;
begin
  DrugReportData:=CallRPCReportText;
  //
  dlgDrugReport.listReport.Items.Clear;
  for Index:=0 to DrugReportData.Count-1 do
  begin
    dlgDrugReport.setReportItem(DrugReportData[Index]);
  end;
  //
  dlgDrugReport.ShowModal;
end;

procedure TdlgDrugDates.btnExportClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    MessageDlg('End date cannot be before start date',mtError,[mbOK],0);
    dtpStartDate.SetFocus;
    Exit;
  end;
  if SaveDialog1.Execute then
  begin
    if not DirectoryExists(ExtractFilePath(SaveDialog1.FileName)) then ForceDirectories(ExtractFilePath(SaveDialog1.FileName));
    ExportReport(SaveDialog1.FileName);
  end;
end;

procedure TdlgDrugDates.btnShowClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    MessageDlg('End date cannot be before start date',mtError,[mbOK],0);
    dtpStartDate.SetFocus;
    Exit;
  end;
  DisplayReport;
end;

procedure TdlgDrugDates.btnPrintClick(Sender: TObject);
begin
  if ((not cbStartDate.Checked) and (not cbEndDate.Checked)) and (getEndDate<getStartDate) then
  begin
    MessageDlg('End date cannot be before start date',mtError,[mbOK],0);
    dtpStartDate.SetFocus;
    Exit;
  end;
  if PrintDialog1.Execute then
  begin
    DrugReportData:=CallRPCReportText;
    PrintReport;
  end;
end;

procedure TdlgDrugDates.FormShow(Sender: TObject);
var
  Yr,Mn,Dy:Word;
  i:Integer;
begin
  setEndDate(WindowsDateToFMDate(Date));
  //Default Date & Time Pickers
  dtpStartDate.MaxDate:=Date;
  //Populate Year ComboBox
  DecodeDate(Date,Yr,Mn,Dy);
  for i:=1990 to Yr+100 do cmbYear.Items.Add(IntToStr(i));
  cmbYear.ItemIndex:=cmbYear.Items.IndexOf(IntToStr(Yr));
  CallRPCGetHandle(RPCHandle);
end;

procedure TdlgDrugDates.btnOKClick(Sender: TObject);
var
  StartDate,EndDate:String;
begin
  GetStartEndDates(StartDate,EndDate);
  CallRPCTaskReport(StartDate,EndDate);
end;

procedure TdlgDrugDates.FormCreate(Sender: TObject);
begin
  RPCHandle:=TRPCHandle.Create;
end;

procedure TdlgDrugDates.FormDestroy(Sender: TObject);
begin
  RPCHandle.Destroy;
end;

end.
